// prvocisla.cpp
// soubor jsem vytvoril podle predlohy z knihy Dalibora Kacmare

#include <iostream.h>
#include <stdio.h>
#include <math.h>
#include "prvocisla_h.h"			// GUIDS a rozhrani
#include "registry.h"				// Registrace

HINSTANCE g_hInstance;
long g_cComponents = 0;
long g_cServerLocks = 0;

class CPrvocisla : public IPrvocisla
{
public:
	// IUnknown
	ULONG __stdcall AddRef();
	ULONG __stdcall Release();
	HRESULT __stdcall QueryInterface(REFIID iid, void** ppv);

	// IDispatch
	HRESULT __stdcall GetTypeInfoCount(UINT* pCountTypeInfo);
	HRESULT __stdcall GetTypeInfo(UINT iTypeInfo, LCID lcid, ITypeInfo** ppITypeInfo);
	HRESULT __stdcall GetIDsOfNames(REFIID riid, LPOLESTR* rgszNames, UINT cNames, LCID lcid, DISPID* rgDispId);
	HRESULT __stdcall Invoke(DISPID dispIdMember, REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS* pDispParams, VARIANT* pVarResult, EXCEPINFO* pExcepInfo, UINT* puArgErr);

	// IPrvocisla
	HRESULT __stdcall JePrvocislo(int cislo,int* ano);
	HRESULT __stdcall DejPrvocislo(int n,int * cislo);
	

	CPrvocisla() : m_cRef(1)
		{
			g_cComponents++;
		}
	~CPrvocisla() { g_cComponents--; }
	HRESULT Init(void);

private:
	ULONG m_cRef;				// pocitadlo referenci
	ITypeInfo* m_pTypeInfo;		// ukazatel na typove informace rozhrani
};

// Metoda vrati, zda cislo je (ano=1) nebo neni (ano=0) prvocislo
HRESULT CPrvocisla::JePrvocislo(int cislo,int*ano)
{
	int m = static_cast<int>(sqrt(cislo));

	int i=2;
	
	while ((i<=m) && (cislo%i != 0))
	{
		i++;
	}

	if (i<=m) *ano=0;
	else *ano=1;

	return S_OK;
}

// Metoda vrati n-te prvocislo
HRESULT CPrvocisla::DejPrvocislo(int n,int* cislo)
{
	int poc = 0;
	int i=1;
	int ano;
	while (poc < n)
	{
		i++;
		JePrvocislo(i,&ano);
		if (ano) poc++;
	}

	*cislo = i;

	return S_OK;
}

HRESULT CPrvocisla::GetTypeInfoCount(UINT* pCountTypeInfo)
{
	*pCountTypeInfo = 1;
	return S_OK;
}

HRESULT CPrvocisla::GetTypeInfo(UINT iTypeInfo, LCID lcid, ITypeInfo** ppITypeInfo)
{
	*ppITypeInfo = NULL;
	if(iTypeInfo != 0)
		return DISP_E_BADINDEX;
	m_pTypeInfo->AddRef();
	*ppITypeInfo = m_pTypeInfo;
	return S_OK;
}

HRESULT CPrvocisla::GetIDsOfNames(REFIID riid, LPOLESTR* rgszNames, UINT cNames, LCID lcid, DISPID* rgDispId)
{
	if(riid != IID_NULL)
		return DISP_E_UNKNOWNINTERFACE;
	return DispGetIDsOfNames(m_pTypeInfo, rgszNames, cNames, rgDispId);
}

HRESULT CPrvocisla::Invoke(DISPID dispIdMember, REFIID riid, LCID lcid, WORD wFlags, DISPPARAMS* pDispParams, VARIANT* pVarResult, EXCEPINFO* pExcepInfo, UINT* puArgErr)
{
	if(riid != IID_NULL)
		return DISP_E_UNKNOWNINTERFACE;
	return DispInvoke(this, m_pTypeInfo, dispIdMember, wFlags, pDispParams, pVarResult, pExcepInfo, puArgErr); 
}

HRESULT CPrvocisla::Init(void)
{
	ITypeLib* pTypeLib;
	if(FAILED(LoadRegTypeLib(LIBID_Prvocisla, 1, 0, LANG_NEUTRAL, &pTypeLib)))
		return E_FAIL;
	HRESULT hr = pTypeLib->GetTypeInfoOfGuid(IID_IPrvocisla, &m_pTypeInfo);
	pTypeLib->Release();
	return hr;
}

ULONG CPrvocisla::AddRef()
{
	return ++m_cRef;
}

ULONG CPrvocisla::Release()
{
	if(--m_cRef != 0)
		return m_cRef;
	m_pTypeInfo->Release();
	delete this;
	return 0;
}

HRESULT CPrvocisla::QueryInterface(REFIID riid, void** ppv)
{
	if(riid == IID_IUnknown)
		*ppv = (IUnknown*)this;
	else if(riid == IID_IPrvocisla)
		*ppv = (IPrvocisla*)this;
	else if(riid == IID_IDispatch)	// dereferujeme na komponentu
		*ppv = (IDispatch*)this;
	else 
	{
		*ppv = NULL;
		return E_NOINTERFACE;
	}
	AddRef();
	return S_OK;
}

class CFactory : public IClassFactory
{
public:
	// IUnknown
	ULONG __stdcall AddRef();
	ULONG __stdcall Release();
	HRESULT __stdcall QueryInterface(REFIID iid, void** ppv);

	// IClassFactory
	HRESULT __stdcall CreateInstance(IUnknown *pUnknownOuter, REFIID iid, void** ppv);
	HRESULT __stdcall LockServer(BOOL bLock);

	CFactory() : m_cRef(1) { }
	~CFactory() { }

private:
	long m_cRef;
};

ULONG CFactory::AddRef()
{
	return ++m_cRef;
}

ULONG CFactory::Release()
{
	if(--m_cRef != 0)
		return m_cRef;
	delete this;
	return 0;
}

HRESULT CFactory::QueryInterface(REFIID iid, void** ppv)
{
	if((iid == IID_IUnknown) || (iid == IID_IClassFactory))
		*ppv = (IClassFactory *)this;
	else
	{
		*ppv = NULL;
		return E_NOINTERFACE;
	}
	AddRef();
	return S_OK;
}

HRESULT CFactory::CreateInstance(IUnknown *pUnknownOuter, REFIID iid, void** ppv)
{
	if(pUnknownOuter != NULL)
		return CLASS_E_NOAGGREGATION;

	CPrvocisla *pPrvocisla = new CPrvocisla;

	if(pPrvocisla == NULL)
		return E_OUTOFMEMORY;

	// Inicializace vnitrniho agregovaneho objektu
	pPrvocisla->Init();

	HRESULT hr = pPrvocisla->QueryInterface(iid, ppv);

	pPrvocisla->Release();
	return hr;
}

HRESULT CFactory::LockServer(BOOL bLock)
{
	if(bLock)
		g_cServerLocks++;
	else
		g_cServerLocks--;
	return S_OK;
}

HRESULT __stdcall DllCanUnloadNow()
{
	if(g_cServerLocks == 0 && g_cComponents == 0)
		return S_OK;
	else
		return S_FALSE;
}

HRESULT __stdcall DllGetClassObject(REFCLSID clsid, REFIID iid, void** ppv)
{
	if(clsid != CLSID_Prvocisla)
		return CLASS_E_CLASSNOTAVAILABLE;

	CFactory* pFactory = new CFactory;
	if(pFactory == NULL)
		return E_OUTOFMEMORY;

	HRESULT hr = pFactory->QueryInterface(iid, ppv);
	pFactory->Release();
	return hr;
}

HRESULT __stdcall DllRegisterServer()
{
	char DllPath[256];
	OLECHAR wDllPath[256];
	GetModuleFileName(g_hInstance, DllPath, 256);
	mbstowcs(wDllPath, DllPath, 256);
	ITypeLib* pTypeLib;
	HRESULT hr = LoadTypeLibEx(wDllPath, REGKIND_REGISTER, &pTypeLib);
	if(FAILED(hr))
		return hr;
	pTypeLib->Release();

	return RegisterServer("Prvocislo.dll", CLSID_Prvocisla,
		"Komponenta PRVOCISLA",
		"Prvocisla.Prvocisla", "Prvocisla.Prvocisla.1", NULL);
}

HRESULT __stdcall DllUnregisterServer()
{
	UnRegisterTypeLib(LIBID_Prvocisla, 1, 0, LANG_NEUTRAL, SYS_WIN32);
	return UnregisterServer(CLSID_Prvocisla, "Prvocisla.Prvocisla", "Prvocisla.Prvocisla.1");
}

BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD dwReason, void* pv)
{
	g_hInstance = hInstance;
	return TRUE;
}